import os, sys
import traceback

import xml.etree.ElementTree as xmlet
import numpy as np
import csv

from PyQt5.QtCore import pyqtSignal, QPointF, Qt, QCoreApplication
from PyQt5 import QtGui
import PyQt5.QtWidgets as qt
from PyQt5 import QtCore

import UsefullVariables as vrb
import UsefullWidgets as wgt
import UsefullFunctions as fct
import DatabaseFunction as Dfct
import UsefullDisplay as Dsp

from HistogramWidget import HistogramWidget

import PyIPSDK
import PyIPSDK.IPSDKIPLMorphology as morpho
import PyIPSDK.IPSDKIPLAdvancedMorphology as advmorpho
import PyIPSDK.IPSDKIPLColor as color
import PyIPSDK.IPSDKIPLUtility as util
import PyIPSDK.IPSDKIPLFiltering as filtering
import PyIPSDK.IPSDKIPLBinarization as bin
import PyIPSDK.IPSDKIPLArithmetic as arithm
import PyIPSDK.IPSDKIPLGlobalMeasure as glbmsr
import PyIPSDK.IPSDKIPLGeometricTransform as gtrans
import PyIPSDK.IPSDKIPLShapeAnalysis as shapeanalysis
import PyIPSDK.IPSDKIPLShapeSegmentation as shapesegmentation
import PyIPSDK.IPSDKIPLIntensityTransform as itrans
import PyIPSDK.IPSDKIPLLogical as logic
import PyIPSDK.IPSDKIPLFeatureDetection as fd
import PyIPSDK.IPSDKIPLStats as stats
import PyIPSDK.IPSDKIPLClassification as classif
import PyIPSDK.IPSDKIPLRegistration as registration
import PyIPSDK.IPSDKIPLMachineLearning as ml

import PyIPSDK.IPSDKUI as ui

mainWindow = vrb.mainWindow

try:
    groupMenu = mainWindow.groupMenu

    buttonMultifiles = wgt.PushButtonImage(vrb.folderMacroInterface + "/Macro_Save_Multifiles/Folder_Save.png")
    buttonMultifiles.setFixedSize(30 * vrb.ratio, 30 * vrb.ratio)
    buttonMultifiles.setToolTip("Save current image as multi files")

    groupMenu.layoutBar1.addWidget(buttonMultifiles, 0, vrb.numMacro, Qt.AlignVCenter)
    vrb.numMacro += 1
except:
    pass

class InformationWidget(qt.QWidget):

    def __init__(self):
        qt.QWidget.__init__(self)

        try:
            self.setWindowFlag(Qt.WindowStaysOnTopHint)
        except:
            self.setWindowFlags(Qt.WindowStaysOnTopHint)

        self.stopProcess = False

        self.fileSelector = FileSelector()
        self.fileSelector.setFixedHeight(30 * vrb.ratio)

        self.labelPrefix = qt.QLabel("Prefix")
        self.labelPrefix.setFixedWidth(20*vrb.ratio)
        self.lineEditPrefix = qt.QLineEdit()
        self.lineEditPrefix.setPlaceholderText("Image_")
        self.lineEditPrefix.setFixedWidth(70*vrb.ratio)

        groupBoxPrefix = qt.QGroupBox()
        layoutPrefix = qt.QHBoxLayout()
        layoutPrefix.setContentsMargins(0,0,0,0)
        layoutPrefix.addWidget(self.labelPrefix)
        layoutPrefix.addWidget(self.lineEditPrefix,Qt.AlignLeft)
        groupBoxPrefix.setLayout(layoutPrefix)
        groupBoxPrefix.setFixedWidth(150*vrb.ratio)

        self.buttonValidate = wgt.PushButtonImage(vrb.folderImages + "/Validate.png")
        self.buttonValidate.setFixedSize(25 * vrb.ratio, 25 * vrb.ratio)

        self.layout = qt.QGridLayout()
        self.layout.addWidget(self.fileSelector, 0,0,)
        self.layout.addWidget(groupBoxPrefix, 1,0)
        self.layout.addWidget(self.buttonValidate, 2, 1 ,Qt.AlignRight)

        self.setLayout(self.layout)

        self.fileSelector.buttonOpen.clicked.connect(self.openQDialog)
        self.buttonValidate.clicked.connect(self.startProcess)
        vrb.mainWindow.groupBoxProcessing.buttonStop.clicked.connect(self.stopProcessFunction)

        style = fct.getStyleSheet()
        self.setStyleSheet(style)

        self.setWindowTitle("Save multi files")
        self.setFixedSize(250 * vrb.ratio, 110 * vrb.ratio)

    def openQDialog(self):

        defaultFolder = Dfct.childText(vrb.userPathElement, "CustomImport")
        if defaultFolder is None or defaultFolder == "" or os.path.exists(defaultFolder) == False:
            defaultFolder = "C:/"
        filename = qt.QFileDialog.getExistingDirectory(self, "Select your folder", defaultFolder)
        if filename != "" and filename != None:
            try:
                self.fileSelector.lineEditFile.setText(filename)
                Dfct.SubElement(vrb.userPathElement, "CustomImport").text = os.path.dirname(filename)
                Dfct.saveXmlElement(vrb.userPathElement, vrb.folderInformation + "/UserPath.mho", forceSave=True)
                # self.loadSettings()
            except:
                pass

    def startProcess(self):

        self.stopProcess = False

        self.close()

        try:
            vrb.mainWindow.toggleGroupBoxProcessing()
            vrb.mainWindow.groupBoxProcessing.setText('')
            vrb.mainWindow.groupBoxProcessing.buttonStop.setVisible(True)
            vrb.mainWindow.groupBoxProcessing.progressBar.setVisible(False)
            vrb.mainWindow.groupBoxProcessing.labelLeft.setFixedWidth(190 * vrb.ratio)
        except:
            pass

        try:
            #Get the current image
            image = vrb.mainWindow.currentLabel.image

            #Loop on each plan of the image

            nbCharacters = len(str(image.getSizeZ()))

            for z in range(image.getSizeZ()):
                if self.stopProcess == False:

                    try:
                        vrb.mainWindow.groupBoxProcessing.setText("Saving image (" +str(z+1) + "/" + str(image.getSizeZ()) + ")")
                        qt.QApplication.processEvents()
                    except:
                        pass

                    if image.getSizeC() == 1:
                        plan = PyIPSDK.extractPlan(z, 0, 0, image)
                    else:
                        plan = PyIPSDK.extractColor(z, 0, image)
                    plan = util.copyImg(plan)
                    prefix = self.lineEditPrefix.text()
                    if prefix is None or prefix == "":
                        prefix = self.lineEditPrefix.placeholderText()
                    numPlan = str(z+1)
                    while len(numPlan)<nbCharacters:
                        numPlan = "0"+numPlan

                    PyIPSDK.saveTiffImageFile(self.fileSelector.lineEditFile.text() + "/" + prefix + numPlan + ".tif",plan)

        except:
            traceback.print_exc(file=sys.stderr)
            self.messageBox = wgt.MessageBox("Can't save the current image in the following folder : "+self.fileSelector.lineEditFile.text(), '', buttons=[qt.QMessageBox.Ok], icon=qt.QMessageBox.Warning, windowTitle="Error")
            self.messageBox.exec()

        try:
            vrb.mainWindow.groupBoxProcessing.buttonStop.setVisible(False)
        except:
            pass

        try:
            vrb.mainWindow.toggleGroupBoxProcessing()
        except:
            pass

    def stopProcessFunction(self):

        self.stopProcess = True


class FileSelector(qt.QGroupBox):

    def __init__(self):
        qt.QGroupBox.__init__(self)

        self.labelFile = qt.QLabel("Result folder")
        self.lineEditFile = qt.QLineEdit()
        self.buttonOpen = wgt.PushButtonImage(vrb.folderImages + "/Folder.png")
        self.buttonOpen.setFixedSize(20 * vrb.ratio, 20 * vrb.ratio)

        self.layout = qt.QGridLayout()
        self.layout.addWidget(self.labelFile, 0, 0)
        self.layout.addWidget(self.lineEditFile, 0, 1)
        self.layout.addWidget(self.buttonOpen, 0, 2)
        self.setLayout(self.layout)

        self.layout.setContentsMargins(0, 0, 0, 0)

        style = fct.getStyleSheet()
        self.setStyleSheet(style)

informationWidget = InformationWidget()

def openWidget():
    fct.showWidget(informationWidget)

try:
    buttonMultifiles.clicked.connect(openWidget)
except:
    pass


if __name__ == '__main__':
    import sys

    app = QtCore.QCoreApplication.instance()
    if app is None:
        app = qt.QApplication([])

    sys._excepthook = sys.excepthook


    def exception_hook(exctype, value, traceback):
        print(exctype, value, traceback)
        sys._excepthook(exctype, value, traceback)
        sys.exit(1)


    sys.excepthook = exception_hook

    foo = InformationWidget()
    foo.resize(400,600)
    foo.show()

    app.exec_()